package calculator.presentation;

// Standard imports
import java.math.BigDecimal;
import com.lutris.appserver.server.session.*;
import calculator.CalcApp;
import java.io.IOException;

// Enhydra SuperServlet imports
import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.util.KeywordValueException;
import org.w3c.dom.html.HTMLImageElement;
import org.w3c.dom.Node;

public class CalculatorPresentation implements HttpPresentation {

  private static CalculatorHTML calculator;

  public void run(HttpPresentationComms comms) throws HttpPresentationException, IOException {

    try{
      processButton(comms);
      getState(comms);
    }catch(Exception e){
      e.printStackTrace();
    }
  }

  /**
   *  A sample Enhydra application. A simple calculator is simulated using
   *  a single presentation object. Every time a button is pressed this
   *  presentation object is notified via the querry string
   *  "?button=...". <P>
   *
   *  The responsability of this object is to emit an HTML string that
   *  shows the current state of the calculator. If a button is being
   *  passed to us in the query string, we process that button first,
   *  then show the current state of the calculator. <P>
   *
   *  Because the SessionData object is being used to store the state of
   *  the calculator, multiple users may  use the calculator simultaneously
   *  "for free". <P>
   *
   *  This method is called to process a button push. If no button
   *  was sent in the URL, do nothing.
   *
   *  @author Andy John andy@enhydra.org
   */

  static public void processButton(HttpPresentationComms comms) throws Exception {
    // Get the session data for the user who issued this request.
    SessionData sd = comms.session.getSessionData();
    String button =  (String) comms.request.getParameter("button");

    if (button != null) {
      // Increment the counter stored in the application object.
      CalcApp myApplication = (calculator.CalcApp) comms.application;
      myApplication.buttonsPushed++;

      if (button.equals("0") || button.equals("1") ||
          button.equals("2") || button.equals("3") ||
          button.equals("4") || button.equals("5") ||
          button.equals("6") || button.equals("7") ||
          button.equals("8") || button.equals("9"))
      {
        try{
          addDigit(sd, button);
        }catch(Exception e){
          e.printStackTrace();
        }
      }
      else if (button.equals("point"))
      {
        try{
          addPoint(sd);
        }catch(Exception e){
          e.printStackTrace();
        }
      }
      else if (button.equals("negate"))
      {
        try{
          negate(sd);
        }catch(Exception e){
          e.printStackTrace();
        }
      }
      else if (button.equals("clear")) {
        try{
          sd.set("digits", "0");
          sd.remove("pending_op");
          sd.set("pending_num", new Float(0.0));
          sd.set("overwrite", Boolean.TRUE);
        }catch(Exception e){
          e.printStackTrace();
        }
        } else if (button.equals("equals"))
        {
          try{
            doEquals(sd);
          }catch(Exception e){
            e.printStackTrace();
          }
        }
        else if (button.equals("plus"))
        {
          try{
            doFunction(sd, "+");
          }catch(Exception e){
            e.printStackTrace();
          }
        }
        else if (button.equals("minus"))
        {
          try{
            doFunction(sd, "-");
          }catch(Exception e){
            e.printStackTrace();
          }
        }
        else if (button.equals("times"))
        {
          try{
            doFunction(sd, "*");
          }catch(Exception e){
            e.printStackTrace();
          }
        }
        else if (button.equals("divide"))
        {
          try{
            doFunction(sd, "/");
          }catch(Exception e){
            e.printStackTrace();
          }
        }
    }
  }

  /**
   *  Add a digit to the current display. We might be looking at a
   *  "stale" number (if overwrite is set). If so, then clear the
   *  display and save the current number for later use.
   */
  private static void addDigit(SessionData sd, String button) throws KeywordValueException {
    String digits = (String) sd.get("digits");
    if (digits == null){
      digits = "0";
    }
    if (sd.get("overwrite") != null) {
      try {
        sd.set("pending_num", new Float(digits));
      } catch (NumberFormatException e) {
      }
      digits = "";
      sd.remove("overwrite");
    }
    digits += button;
    sd.set("digits", digits);
  }


  /**
   *  Add a decimal point to the current display. If it already has a
   *  decimal point, then do nothing.
   */
  private static void addPoint(SessionData sd)
      throws KeywordValueException {
    String digits = (String) sd.get("digits");
    if (digits == null)
      digits = "0";
    if (sd.get("overwrite") != null) {
      try {
        sd.set("pending_num", new Float(digits));
      } catch (NumberFormatException e) {
      }
      digits = "";
      sd.remove("overwrite");
    }
    if (digits.indexOf(".") != -1)
      return;
    digits += ".";
    sd.set("digits", digits);
  }

  /**
   *  Flip the sign of the number currently in the display.
   */
  private static void negate(SessionData sd)
      throws KeywordValueException {
    String digits = (String) sd.get("digits");
    if (digits == null)
      return;
    if (digits.startsWith("-"))
      digits = digits.substring(1);
    else
      digits = "-" + digits;
    sd.set("digits", digits);
  }

  /**
   *  If there is a "pending number" (not in the current display),
   *  and there is a "pending operation" (add, subtract etc...),
   *  then use them and the current contents of the display and do some
   *  math. Leave the result in the current display.
   */
  private static void doEquals(SessionData sd)
      throws KeywordValueException {
    String op = (String) sd.get("pending_op");
    if (op == null)
      return;
    Float f = (Float) sd.get("pending_num");
    if (f == null)
      f = new Float("0.0");
    float f2 = f.floatValue();

    String digits = (String) sd.get("digits");
    if (digits == null)
      digits = "0";
    Float f3 = new Float(0);
    try {
      f3 = new Float(digits);
    } catch (NumberFormatException e) {
    }
    float f4 = f3.floatValue();
    double result = 0.0;

    if (op.equals("+")) {
      result = f2 + f4;
    } else if (op.equals("-")) {
      result = f2 - f4;
    } else if (op.equals("*")) {
      result = f2 * f4;
    } else if (op.equals("/")) {
      result = f2 / f4;
    }

    BigDecimal bd = new BigDecimal(result);
    sd.set("digits", bd.toString());
    sd.remove("pending_op");
    sd.remove("pending_num");
    sd.set("overwrite", Boolean.TRUE);
  }

  /**
   *  Remember that a math function was pushed.
   */
  private static void doFunction(SessionData sd, String op)
      throws KeywordValueException {
    if (sd.get("pending_num") != null) {
      doEquals(sd);
    } else {
      String digits = (String) sd.get("digits");
      if (digits == null)
        digits = "0";
    }
    sd.set("pending_op", op);
    sd.set("overwrite", Boolean.TRUE);
  }

  /**
   *  This method initializes an XMLC field with the contents of
   *  the current number in the display. The string of digits is converted
   *  into a sequence of HTML image tags.
   *
   *  This is only a demo program. It does not handle exceptional cases
   *  well, like dividing by zero, overflow, or infinties. The point is to
   *  show off one way to use Enhydra, not be a full-on HP calculator.
   */
  public static void getState(HttpPresentationComms comms) throws Exception {
    // Convert current state to HTML
    SessionData sd = comms.session.getSessionData();
    String digits = (String) sd.get("digits");

    if (digits == null)
      digits = "0";
    // If it's an integer, don't show the decimal point.
    if (digits.endsWith(".0"))
      digits = digits.substring(0, digits.length()-2);
    // If it's too long to show, trim it or just show 0.
    // An error indication would be nice to add (hint hint... :)
    if (digits.indexOf(".") != -1) {
      if (digits.length() > 8)
        digits = digits.substring(0,8); // Truncate
    } else {
      if (digits.length() > 7)
        digits = "0"; // Overflow.
    }

    // Convert the string of digits to HTML.
    String digitHtml = "";
    for (int i=0; i<digits.length(); i++) {
      char c = digits.charAt(i);
      if (c == '0')
        digitHtml += "<img src=media/zero.jpeg>";
      else if (c == '1')
        digitHtml += "<img src=media/one.jpeg>";
      else if (c == '2')
        digitHtml += "<img src=media/two.jpeg>";
      else if (c == '3')
        digitHtml += "<img src=media/three.jpeg>";
      else if (c == '4')
        digitHtml += "<img src=media/four.jpeg>";
      else if (c == '5')
        digitHtml += "<img src=media/five.jpeg>";
      else if (c == '6')
        digitHtml += "<img src=media/six.jpeg>";
      else if (c == '7')
        digitHtml += "<img src=media/seven.jpeg>";
      else if (c == '8')
        digitHtml += "<img src=media/eight.jpeg>";
      else if (c == '9')
        digitHtml += "<img src=media/nine.jpeg>";
      else if (c == '-')
        digitHtml += "<img src=media/dash.jpeg>";
      else if (c == '.')
        digitHtml += "<img src=media/point.jpeg>";
    }
    if (digitHtml.length() == 0)
      digitHtml = "<img src=media/zero.jpeg>";

    // Make the result available to the HTML page.
    calculator = (CalculatorHTML)comms.xmlcFactory.create(CalculatorHTML.class);
    HTMLImageElement digitsDummy = calculator.getElementDigits();
    Node parent = digitsDummy.getParentNode();

    int resultStart = 0;
    int resultEnd = 0;
    while((resultStart=digitHtml.indexOf("<")) != -1){
      resultEnd = digitHtml.indexOf(">");
      String txt = digitHtml.substring(resultStart+9, resultEnd);
      digitHtml = digitHtml.substring(resultEnd+1);

      HTMLImageElement image = (HTMLImageElement) digitsDummy.cloneNode(false);
      image.setSrc(txt);
      parent.appendChild(image);
    }
    parent.removeChild(digitsDummy);
    comms.response.writeDOM(calculator);
  }
}




